@@ -1,7 +1,9 @@
 """Database wrapper."""
 
+import asyncio
 from itertools import chain
 import logging
+from time import perf_counter
 from typing import (
     Any,
     Dict,
@@ -13,16 +15,21 @@ from typing import (
     Union,
 )
 
-import sqlalchemy
-from sqlalchemy.engine import (
-    Connection,
-    ResultProxy,
+from croniter import croniter
+from sqlalchemy import (
+    create_engine,
+    event,
+    text,
 )
 from sqlalchemy.exc import (
     ArgumentError,
     NoSuchModuleError,
 )
 from sqlalchemy_aio import ASYNCIO_STRATEGY
+from sqlalchemy_aio.base import (
+    AsyncConnection,
+    AsyncResultProxy,
+)
 from sqlalchemy_aio.engine import AsyncioEngine
 
 # label used to tag metrics by database
@@ -65,6 +72,13 @@ class InvalidQueryParameters(Exception):
         )
 
 
+class InvalidQuerySchedule(Exception):
+    """Query schedule is wrong or both schedule and interval specified."""
+
+    def __init__(self, query_name: str, message: str):
+        super().__init__(f'Invalid schedule for query "{query_name}": {message}')
+
+
 # database errors that mean the query won't ever succeed.  Not all possible
 # fatal errors are tracked here, because some DBAPI errors can happen in
 # circumstances which can be fatal or not.  Since there doesn't seem to be a
@@ -85,11 +99,14 @@ class QueryResults(NamedTuple):
 
     keys: List[str]
     rows: List[Tuple]
+    latency: Optional[float] = None
 
     @classmethod
-    async def from_results(cls, results: ResultProxy):
+    async def from_results(cls, results: AsyncResultProxy):
         """Return a QueryResults from results for a query."""
-        return cls(await results.keys(), await results.fetchall())
+        conn_info = results._result_proxy.connection.info
+        latency = conn_info.get("query_latency", None)
+        return cls(await results.keys(), await results.fetchall(), latency=latency)
 
 
 class MetricResult(NamedTuple):
@@ -100,34 +117,51 @@ class MetricResult(NamedTuple):
     labels: Dict[str, str]
 
 
+class MetricResults(NamedTuple):
+    """Collection of metric results for a query."""
+
+    results: List[MetricResult]
+    latency: Optional[float] = None
+
+
 class Query:
     """Query definition and configuration."""
 
     def __init__(
         self,
         name: str,
-        interval: int,
         databases: List[str],
         metrics: List[QueryMetric],
         sql: str,
         parameters: Optional[Dict[str, Any]] = None,
+        interval: Optional[int] = None,
+        schedule: Optional[str] = None,
+        config_name: Optional[str] = None,
     ):
         self.name = name
-        self.interval = interval
         self.databases = databases
         self.metrics = metrics
         self.sql = sql
         self.parameters = parameters or {}
-        self._check_parameters()
+        self.interval = interval
+        self.schedule = schedule
+        self.config_name = config_name or name
+        self._check_schedule()
+        self._check_query_parameters()
+
+    @property
+    def timed(self) -> bool:
+        """Whether the query is run periodically via interval or schedule."""
+        return bool(self.interval or self.schedule)
 
     def labels(self) -> FrozenSet[str]:
         """Resturn all labels for metrics in the query."""
         return frozenset(chain(*(metric.labels for metric in self.metrics)))
 
-    def results(self, query_results: QueryResults) -> List[MetricResult]:
+    def results(self, query_results: QueryResults) -> MetricResults:
         """Return MetricResults from a query."""
         if not query_results.rows:
-            return []
+            return MetricResults([])
 
         result_keys = sorted(query_results.keys)
         labels = self.labels()
@@ -147,10 +181,19 @@ class Query:
                     {label: values[label] for label in metric.labels},
                 )
                 results.append(metric_result)
-        return results
 
-    def _check_parameters(self):
-        expr = sqlalchemy.text(self.sql)
+        return MetricResults(results, latency=query_results.latency)
+
+    def _check_schedule(self):
+        if self.interval and self.schedule:
+            raise InvalidQuerySchedule(
+                self.name, "both interval and schedule specified"
+            )
+        if self.schedule and not croniter.is_valid(self.schedule):
+            raise InvalidQuerySchedule(self.name, "invalid schedule format")
+
+    def _check_query_parameters(self):
+        expr = text(self.sql)
         query_params = set(expr.compile().params)
         if set(self.parameters) != query_params:
             raise InvalidQueryParameters(self.name)
@@ -160,7 +203,7 @@ class DataBase:
     """A database to perform Queries."""
 
     _engine: AsyncioEngine
-    _conn: Optional[Connection] = None
+    _conn: Optional[AsyncConnection] = None
     _logger: logging.Logger = logging.getLogger()
     _pending_queries: int = 0
 
@@ -168,24 +211,34 @@ class DataBase:
         self,
         name: str,
         dsn: str,
+        connect_sql: Optional[List[str]] = None,
         keep_connected: Optional[bool] = True,
+        autocommit: Optional[bool] = True,
         labels: Optional[Dict[str, str]] = None,
     ):
         self.name = name
         self.dsn = dsn
+        self.connect_sql = connect_sql or []
         self.keep_connected = keep_connected
+        self.autocommit = autocommit
         self.labels = labels or {}
+        self._connect_lock = asyncio.Lock()
         try:
-            self._engine = sqlalchemy.create_engine(
-                dsn, strategy=ASYNCIO_STRATEGY, execution_options={"autocommit": True},
+            self._engine = create_engine(
+                dsn,
+                strategy=ASYNCIO_STRATEGY,
+                execution_options={"autocommit": self.autocommit},
             )
         except ImportError as error:
             raise self._db_error(f'module "{error.name}" not found', fatal=True)
         except (ArgumentError, ValueError, NoSuchModuleError):
             raise self._db_error(f'Invalid database DSN: "{self.dsn}"', fatal=True)
 
+        self._setup_query_latency_tracking()
+
     async def __aenter__(self):
-        return await self.connect()
+        await self.connect()
+        return self
 
     async def __aexit__(self, exc_type, exc_value, traceback):
         await self.close()
@@ -201,30 +254,36 @@ class DataBase:
 
     async def connect(self):
         """Connect to the database."""
-        try:
-            self._conn = await self._engine.connect()
-        except Exception as error:
-            raise self._db_error(error)
-        self._logger.debug(f'connected to database "{self.name}"')
-        return self
+        async with self._connect_lock:
+            if self.connected:
+                return
+
+            try:
+                self._conn = await self._engine.connect()
+            except Exception as error:
+                raise self._db_error(error)
+
+            self._logger.debug(f'connected to database "{self.name}"')
+            for sql in self.connect_sql:
+                try:
+                    await self.execute_sql(sql)
+                except Exception as error:
+                    await self._close()
+                    raise self._db_error(f'failed executing query "{sql}": {error}')
 
     async def close(self):
         """Close the database connection."""
-        if not self.connected:
-            return
-        await self._conn.close()
-        self._conn = None
-        self._pending_queries = 0
-        self._logger.debug(f'disconnected from database "{self.name}"')
+        async with self._connect_lock:
+            if not self.connected:
+                return
+            await self._close()
 
-    async def execute(self, query: Query) -> List[MetricResult]:
+    async def execute(self, query: Query) -> MetricResults:
         """Execute a query."""
-        if not self.connected:
-            await self.connect()
-
+        await self.connect()
         self._logger.debug(f'running query "{query.name}" on database "{self.name}"')
         self._pending_queries += 1
-        self._conn: Connection
+        self._conn: AsyncConnection
         try:
             result = await self._execute_query(query)
             return query.results(await QueryResults.from_results(result))
@@ -240,22 +299,45 @@ class DataBase:
 
     async def execute_sql(
         self, sql: str, parameters: Optional[Dict[str, Any]] = None
-    ) -> ResultProxy:
+    ) -> AsyncResultProxy:
         """Execute a raw SQL query."""
         if parameters is None:
             parameters = {}
-        self._conn: Connection
-        return await self._conn.execute(sqlalchemy.text(sql), parameters)
+        self._conn: AsyncConnection
+        return await self._conn.execute(text(sql), parameters)
 
-    async def _execute_query(self, query: Query) -> ResultProxy:
+    async def _execute_query(self, query: Query) -> AsyncResultProxy:
         """Execute a query."""
         return await self.execute_sql(query.sql, parameters=query.parameters)
 
+    async def _close(self):
+        await self._conn.close()
+        self._conn = None
+        self._pending_queries = 0
+        self._logger.debug(f'disconnected from database "{self.name}"')
+
+    def _setup_query_latency_tracking(self):
+        engine = self._engine.sync_engine
+
+        @event.listens_for(engine, "before_cursor_execute")
+        def before_cursor_execute(
+            conn, cursor, statement, parameters, context, executemany
+        ):
+            conn.info["query_start_time"] = perf_counter()
+
+        @event.listens_for(engine, "after_cursor_execute")
+        def after_cursor_execute(
+            conn, cursor, statement, parameters, context, executemany
+        ):
+            conn.info["query_latency"] = perf_counter() - conn.info.pop(
+                "query_start_time"
+            )
+
     def _query_db_error(
         self, query_name: str, error: Union[str, Exception], fatal: bool = False,
     ):
         """Create and log a DataBaseError for a failed query."""
-        message = str(error).strip()
+        message = self._error_message(error)
         self._logger.error(
             f'query "{query_name}" on database "{self.name}" failed: ' + message
         )
@@ -263,6 +345,13 @@ class DataBase:
 
     def _db_error(self, error: Union[str, Exception], fatal: bool = False):
         """Create and log a DataBaseError."""
-        message = str(error).strip()
+        message = self._error_message(error)
         self._logger.error(f'error from database "{self.name}": {message}')
         return DataBaseError(message, fatal=fatal)
+
+    def _error_message(self, error: Union[str, Exception]) -> str:
+        """Return a message from an error."""
+        message = str(error).strip()
+        if not message and isinstance(error, Exception):
+            message = error.__class__.__name__
+        return message
