@@ -7,6 +7,7 @@ import pytest
 from ..config import (
     ConfigError,
     DB_ERRORS_METRIC_NAME,
+    GLOBAL_METRICS,
     load_config,
     QUERIES_METRIC_NAME,
 )
@@ -118,7 +119,11 @@ class TestLoadConfig:
         config = {
             "databases": {
                 "db1": {"dsn": "sqlite:///foo"},
-                "db2": {"dsn": "sqlite:///bar", "keep-connected": False},
+                "db2": {
+                    "dsn": "sqlite:///bar",
+                    "keep-connected": False,
+                    "autocommit": False,
+                },
             },
             "metrics": {},
             "queries": {},
@@ -132,9 +137,13 @@ class TestLoadConfig:
         assert database1.name == "db1"
         assert database1.dsn == "sqlite:///foo"
         assert database1.keep_connected
+        assert database1.autocommit
+        assert database1._engine._execution_options == {"autocommit": True}
         assert database2.name == "db2"
         assert database2.dsn == "sqlite:///bar"
         assert not database2.keep_connected
+        assert not database2.autocommit
+        assert database2._engine._execution_options == {"autocommit": False}
 
     def test_load_databases_dsn_from_env(self, logger, write_config):
         """The database DSN can be loaded from env."""
@@ -234,6 +243,20 @@ class TestLoadConfig:
             load_config(fd, logger, env={})
         assert str(err.value) == "Not all databases define the same labels"
 
+    def test_load_databases_connect_sql(self, logger, write_config):
+        """Databases can have queries defined to run on connection."""
+        config = {
+            "databases": {
+                "db": {"dsn": "sqlite://", "connect-sql": ["SELECT 1", "SELECT 2"]},
+            },
+            "metrics": {},
+            "queries": {},
+        }
+        config_file = write_config(config)
+        with config_file.open() as fd:
+            result = load_config(fd, logger, env={})
+        assert result.databases["db"].connect_sql == ["SELECT 1", "SELECT 2"]
+
     def test_load_metrics_section(self, logger, write_config):
         """The 'metrics' section is loaded from the config file."""
         config = {
@@ -312,6 +335,17 @@ class TestLoadConfig:
             == 'Labels for metric "m" overlap with reserved/database ones: l1'
         )
 
+    @pytest.mark.parametrize("global_name", list(GLOBAL_METRICS))
+    def test_load_metrics_reserved_name(self, config_full, write_config, global_name):
+        config_full["metrics"][global_name] = {"type": "counter"}
+        config_file = write_config(config_full)
+        with pytest.raises(ConfigError) as err, config_file.open() as fd:
+            load_config(fd, logger)
+        assert (
+            str(err.value)
+            == f'Label name "{global_name} is reserved for builtin metric'
+        )
+
     def test_load_metrics_unsupported_type(self, logger, write_config):
         """An error is raised if an unsupported metric type is passed."""
         config = {
@@ -435,6 +469,52 @@ class TestLoadConfig:
             == 'Parameters for query "q[params0]" don\'t match those from SQL'
         )
 
+    def test_load_queries_section_with_schedule_and_interval(
+        self, logger, write_config
+    ):
+        """An error is raised if query schedule and interval are both present."""
+        config = {
+            "databases": {"db": {"dsn": "sqlite://"}},
+            "metrics": {"m": {"type": "summary"}},
+            "queries": {
+                "q": {
+                    "databases": ["db"],
+                    "metrics": ["m"],
+                    "sql": "SELECT 1",
+                    "interval": 10,
+                    "schedule": "0 * * * *",
+                },
+            },
+        }
+        config_file = write_config(config)
+        with pytest.raises(ConfigError) as err, config_file.open() as fd:
+            load_config(fd, logger)
+        assert (
+            str(err.value)
+            == 'Invalid schedule for query "q": both interval and schedule specified'
+        )
+
+    def test_load_queries_section_invalid_schedule(self, logger, write_config):
+        """An error is raised if query schedule has wrong format."""
+        config = {
+            "databases": {"db": {"dsn": "sqlite://"}},
+            "metrics": {"m": {"type": "summary"}},
+            "queries": {
+                "q": {
+                    "databases": ["db"],
+                    "metrics": ["m"],
+                    "sql": "SELECT 1",
+                    "schedule": "wrong",
+                },
+            },
+        }
+        config_file = write_config(config)
+        with pytest.raises(ConfigError) as err, config_file.open() as fd:
+            load_config(fd, logger)
+        assert (
+            str(err.value) == 'Invalid schedule for query "q": invalid schedule format'
+        )
+
     @pytest.mark.parametrize(
         "config,error_message",
         [
