@@ -11,6 +11,7 @@ from ..db import (
     DataBase,
     DataBaseError,
     InvalidQueryParameters,
+    InvalidQuerySchedule,
     InvalidResultColumnNames,
     InvalidResultCount,
     MetricResult,
@@ -32,7 +33,6 @@ class TestQuery:
         """A query can be instantiated with the specified arguments."""
         query = Query(
             "query",
-            20,
             ["db1", "db2"],
             [
                 QueryMetric("metric1", ["label1", "label2"]),
@@ -41,7 +41,7 @@ class TestQuery:
             "SELECT 1",
         )
         assert query.name == "query"
-        assert query.interval == 20
+        assert query.config_name == "query"
         assert query.databases == ["db1", "db2"]
         assert query.metrics == [
             QueryMetric("metric1", ["label1", "label2"]),
@@ -49,12 +49,23 @@ class TestQuery:
         ]
         assert query.sql == "SELECT 1"
         assert query.parameters == {}
+        assert query.interval is None
+
+    def test_instantiate_with_config_name(self):
+        """A query can be instantiated with a config_name different from name."""
+        query = Query(
+            "query",
+            ["db"],
+            [QueryMetric("metric", [])],
+            "SELECT metric1 FROM table",
+            config_name="query_config",
+        )
+        assert query.config_name == "query_config"
 
     def test_instantiate_with_parameters(self):
         """A query can be instantiated with parameters."""
         query = Query(
             "query",
-            20,
             ["db1", "db2"],
             [
                 QueryMetric("metric1", ["label1", "label2"]),
@@ -71,7 +82,6 @@ class TestQuery:
         with pytest.raises(InvalidQueryParameters):
             Query(
                 "query",
-                20,
                 ["db1", "db2"],
                 [
                     QueryMetric("metric1", ["label1", "label2"]),
@@ -82,11 +92,87 @@ class TestQuery:
                 parameters={"param1": 1, "param2": 2},
             )
 
+    def test_instantiate_with_interval(self):
+        """A query can be instantiated with an interval."""
+        query = Query(
+            "query",
+            ["db1", "db2"],
+            [
+                QueryMetric("metric1", ["label1", "label2"]),
+                QueryMetric("metric2", ["label2"]),
+            ],
+            "SELECT 1",
+            interval=20,
+        )
+        assert query.interval == 20
+
+    def test_instantiate_with_schedule(self):
+        """A query can be instantiated with a schedule."""
+        query = Query(
+            "query",
+            ["db1", "db2"],
+            [
+                QueryMetric("metric1", ["label1", "label2"]),
+                QueryMetric("metric2", ["label2"]),
+            ],
+            "SELECT 1",
+            schedule="0 * * * *",
+        )
+        assert query.schedule == "0 * * * *"
+
+    def test_instantiate_with_interval_and_schedule(self):
+        """Interval and schedule can't be specified together."""
+        with pytest.raises(InvalidQuerySchedule) as error:
+            Query(
+                "query",
+                ["db1"],
+                [QueryMetric("metric1", [])],
+                "SELECT 1",
+                interval=20,
+                schedule="0 * * * *",
+            )
+        assert (
+            str(error.value)
+            == 'Invalid schedule for query "query": both interval and schedule specified'
+        )
+
+    def test_instantiate_with_invalid_schedule(self):
+        """Invalid query schedule raises an error."""
+        with pytest.raises(InvalidQuerySchedule) as error:
+            Query(
+                "query",
+                ["db1"],
+                [QueryMetric("metric1", [])],
+                "SELECT 1",
+                schedule="wrong",
+            )
+        assert (
+            str(error.value)
+            == 'Invalid schedule for query "query": invalid schedule format'
+        )
+
+    @pytest.mark.parametrize(
+        "kwargs,is_timed",
+        [({}, False), ({"interval": 20}, True), ({"schedule": "1 * * * *"}, True)],
+    )
+    def test_timed(self, kwargs, is_timed):
+        """Query.timed reports whether the query is run with a time schedule"""
+        query = Query(
+            "query",
+            ["db1", "db2"],
+            [
+                QueryMetric("metric1", ["label1", "label2"]),
+                QueryMetric("metric2", ["label2"]),
+            ],
+            "SELECT 1",
+            **kwargs,
+        )
+        assert query.timed == is_timed
+
     def test_labels(self):
         """All labels for the query can be returned."""
         query = Query(
             "query",
-            20,
             ["db1", "db2"],
             [
                 QueryMetric("metric1", ["label1", "label2"]),
@@ -98,21 +184,22 @@ class TestQuery:
 
     def test_results_empty(self):
         """No error is raised if the result set is empty"""
-        query = Query("query", 20, ["db"], [QueryMetric("metric", [])], "")
+        query = Query("query", ["db"], [QueryMetric("metric", [])], "")
         query_results = QueryResults(["one"], [])
-        assert query.results(query_results) == []
+        metrics_results = query.results(query_results)
+        assert metrics_results.results == []
 
     def test_results_metrics(self):
         """The results method returns results by matching metrics name."""
         query = Query(
             "query",
-            20,
             ["db"],
             [QueryMetric("metric1", []), QueryMetric("metric2", [])],
             "",
         )
         query_results = QueryResults(["metric2", "metric1"], [(11, 22), (33, 44)])
-        assert query.results(query_results) == [
+        metrics_results = query.results(query_results)
+        assert metrics_results.results == [
             MetricResult("metric1", 22, {}),
             MetricResult("metric2", 11, {}),
             MetricResult("metric1", 44, {}),
@@ -123,7 +210,6 @@ class TestQuery:
         """The results method returns results by matching metrics name."""
         query = Query(
             "query",
-            20,
             ["db"],
             [
                 QueryMetric("metric1", ["label1", "label2"]),
@@ -135,7 +221,8 @@ class TestQuery:
             ["metric2", "metric1", "label2", "label1"],
             [(11, 22, "foo", "bar"), (33, 44, "baz", "bza")],
         )
-        assert query.results(query_results) == [
+        metrics_results = query.results(query_results)
+        assert metrics_results.results == [
             MetricResult("metric1", 22, {"label1": "bar", "label2": "foo"}),
             MetricResult("metric2", 11, {"label2": "foo"}),
             MetricResult("metric1", 44, {"label1": "bza", "label2": "baz"}),
@@ -144,21 +231,21 @@ class TestQuery:
 
     def test_results_wrong_result_count(self):
         """An error is raised if the result column count is wrong."""
-        query = Query("query", 20, ["db"], [QueryMetric("metric1", [])], "")
+        query = Query("query", ["db"], [QueryMetric("metric1", [])], "")
         query_results = QueryResults(["one", "two"], [(1, 2)])
         with pytest.raises(InvalidResultCount):
             query.results(query_results)
 
     def test_results_wrong_result_count_with_label(self):
         """An error is raised if the result column count is wrong."""
-        query = Query("query", 20, ["db"], [QueryMetric("metric1", ["label1"])], "")
+        query = Query("query", ["db"], [QueryMetric("metric1", ["label1"])], "")
         query_results = QueryResults(["one"], [(1,)])
         with pytest.raises(InvalidResultCount):
             query.results(query_results)
 
     def test_results_wrong_names_with_labels(self):
         """An error is raised if metric and labels names don't match."""
-        query = Query("query", 20, ["db"], [QueryMetric("metric1", ["label1"])], "")
+        query = Query("query", ["db"], [QueryMetric("metric1", ["label1"])], "")
         query_results = QueryResults(["one", "two"], [(1, 2)])
         with pytest.raises(InvalidResultColumnNames):
             query.results(query_results)
@@ -174,6 +261,20 @@ class TestQueryResults:
             query_results = await QueryResults.from_results(result)
         assert query_results.keys == ["a", "b"]
         assert query_results.rows == [(1, 2)]
+        assert query_results.latency is None
+
+    @pytest.mark.asyncio
+    async def test_from_results_with_latency(self):
+        """The from_results method creates a QueryResult."""
+        engine = create_engine("sqlite://", strategy=ASYNCIO_STRATEGY)
+        async with engine.connect() as conn:
+            result = await conn.execute("SELECT 1 AS a, 2 AS b")
+            # simulate latency tracking
+            conn.sync_connection.info["query_latency"] = 1.2
+            query_results = await QueryResults.from_results(result)
+        assert query_results.keys == ["a", "b"]
+        assert query_results.rows == [(1, 2)]
+        assert query_results.latency == 1.2
 
 
 @pytest.fixture
@@ -210,12 +311,16 @@ class TestDataBase:
         assert str(error.value) == 'module "psycopg2" not found'
         assert 'module "psycopg2" not found' in caplog.text
 
+    def test_instantiate_autocommit_false(self):
+        """Query autocommit can be set to False."""
+        db = DataBase("db", "sqlite:///foo", autocommit=False)
+        assert not db.autocommit
+
     @pytest.mark.parametrize("dsn", ["foo-bar", "unknown:///db"])
     def test_instantiate_invalid_dsn(self, caplog, dsn):
         """An error is raised if a the provided DSN is invalid."""
-        with caplog.at_level(logging.ERROR):
-            with pytest.raises(DataBaseError) as error:
-                DataBase("db", dsn)
+        with caplog.at_level(logging.ERROR), pytest.raises(DataBaseError) as error:
+            DataBase("db", dsn)
         assert str(error.value) == f'Invalid database DSN: "{dsn}"'
         assert f'Invalid database DSN: "{dsn}"' in caplog.text
 
@@ -236,14 +341,46 @@ class TestDataBase:
         assert isinstance(db._conn, AsyncConnection)
         assert caplog.messages == ['connected to database "db"']
 
+    @pytest.mark.asyncio
+    async def test_connect_lock(self, caplog, db):
+        """The connect method has a lock to prevent concurrent calls."""
+        with caplog.at_level(logging.DEBUG):
+            await asyncio.gather(db.connect(), db.connect())
+        assert caplog.messages == ['connected to database "db"']
+
     @pytest.mark.asyncio
     async def test_connect_error(self):
         """A DataBaseError is raised if database connection fails."""
-        db = DataBase("db", f"sqlite:////invalid")
+        db = DataBase("db", "sqlite:////invalid")
         with pytest.raises(DataBaseError) as error:
             await db.connect()
         assert "unable to open database file" in str(error.value)
 
+    @pytest.mark.asyncio
+    async def test_connect_sql(self):
+        """If connect_sql is specified, it's run at connection."""
+        db = DataBase("db", "sqlite://", connect_sql=["SELECT 1", "SELECT 2"])
+
+        queries = []
+
+        async def execute_sql(sql):
+            queries.append(sql)
+
+        db.execute_sql = execute_sql
+        await db.connect()
+        assert queries == ["SELECT 1", "SELECT 2"]
+        await db.close()
+
+    @pytest.mark.asyncio
+    async def test_connect_sql_fail(self, caplog):
+        """If the SQL at connection fails, an error is raised."""
+        db = DataBase("db", "sqlite://", connect_sql=["WRONG"])
+        with caplog.at_level(logging.DEBUG), pytest.raises(DataBaseError) as error:
+            await db.connect()
+        assert not db.connected
+        assert 'failed executing query "WRONG"' in str(error.value)
+        assert 'disconnected from database "db"' in caplog.messages
+
     @pytest.mark.asyncio
     async def test_close(self, caplog, db):
         """The close method closes database connection."""
@@ -260,7 +397,7 @@ class TestDataBase:
         """A message is logged about the query being executed."""
         db = DataBase("db", "sqlite://")
         query = Query(
-            "query", 20, ["db"], [QueryMetric("metric", [])], "SELECT 1.0 AS metric"
+            "query", ["db"], [QueryMetric("metric", [])], "SELECT 1.0 AS metric"
         )
         await db.connect()
         with caplog.at_level(logging.DEBUG):
@@ -274,7 +411,7 @@ class TestDataBase:
         """If keep_connected is set to true, the db is not closed."""
         db = DataBase("db", "sqlite://", keep_connected=connected)
         query = Query(
-            "query", 20, ["db"], [QueryMetric("metric", [])], "SELECT 1.0 AS metric"
+            "query", ["db"], [QueryMetric("metric", [])], "SELECT 1.0 AS metric"
         )
         await db.connect()
         await db.execute(query)
@@ -286,10 +423,10 @@ class TestDataBase:
         """The db is disconnected only after pending queries are run."""
         db = DataBase("db", "sqlite://", keep_connected=False)
         query1 = Query(
-            "query1", 5, ["db"], [QueryMetric("metric1", [])], "SELECT 1.0 AS metric1"
+            "query1", ["db"], [QueryMetric("metric1", [])], "SELECT 1.0 AS metric1"
         )
         query2 = Query(
-            "query1", 5, ["db"], [QueryMetric("metric2", [])], "SELECT 1.0 AS metric2"
+            "query1", ["db"], [QueryMetric("metric2", [])], "SELECT 1.0 AS metric2"
         )
         await db.connect()
         await asyncio.gather(db.execute(query1), db.execute(query2))
@@ -299,10 +436,10 @@ class TestDataBase:
     async def test_execute_not_connected(self, db):
         """The execute recconnects to the database if not connected."""
         query = Query(
-            "query", 20, ["db"], [QueryMetric("metric", [])], "SELECT 1 AS metric"
+            "query", ["db"], [QueryMetric("metric", [])], "SELECT 1 AS metric"
         )
-        result = await db.execute(query)
-        assert result == [MetricResult("metric", 1, {})]
+        metric_results = await db.execute(query)
+        assert metric_results.results == [MetricResult("metric", 1, {})]
         # the connection is kept for reuse
         assert not db._conn.closed
 
@@ -315,19 +452,19 @@ class TestDataBase:
         )
         query = Query(
             "query",
-            20,
             ["db"],
             [QueryMetric("metric1", []), QueryMetric("metric2", [])],
             sql,
         )
         await db.connect()
-        result = await db.execute(query)
-        assert result == [
+        metric_results = await db.execute(query)
+        assert metric_results.results == [
             MetricResult("metric1", 10, {}),
             MetricResult("metric2", 20, {}),
             MetricResult("metric1", 30, {}),
             MetricResult("metric2", 40, {}),
         ]
+        assert isinstance(metric_results.latency, float)
 
     @pytest.mark.asyncio
     async def test_execute_with_labels(self, db):
@@ -343,7 +480,6 @@ class TestDataBase:
             """
         query = Query(
             "query",
-            20,
             ["db"],
             [
                 QueryMetric("metric1", ["label1", "label2"]),
@@ -352,8 +488,8 @@ class TestDataBase:
             sql,
         )
         await db.connect()
-        result = await db.execute(query)
-        assert result == [
+        metric_results = await db.execute(query)
+        assert metric_results.results == [
             MetricResult("metric1", 22, {"label1": "bar", "label2": "foo"}),
             MetricResult("metric2", 11, {"label2": "foo"}),
             MetricResult("metric1", 44, {"label1": "bza", "label2": "baz"}),
@@ -364,11 +500,7 @@ class TestDataBase:
     async def test_execute_query_invalid_count(self, caplog, db):
         """If the number of fields don't match, an error is raised."""
         query = Query(
-            "query",
-            20,
-            ["db"],
-            [QueryMetric("metric", [])],
-            "SELECT 1 AS metric, 2 AS other",
+            "query", 20, [QueryMetric("metric", [])], "SELECT 1 AS metric, 2 AS other",
         )
         await db.connect()
         with caplog.at_level(logging.ERROR):
@@ -385,11 +517,7 @@ class TestDataBase:
     async def test_execute_query_invalid_count_with_labels(self, db):
         """If the number of fields don't match, an error is raised."""
         query = Query(
-            "query",
-            20,
-            ["db"],
-            [QueryMetric("metric", ["label"])],
-            "SELECT 1 as metric",
+            "query", ["db"], [QueryMetric("metric", ["label"])], "SELECT 1 as metric",
         )
         await db.connect()
         with pytest.raises(DataBaseError) as error:
@@ -402,7 +530,6 @@ class TestDataBase:
         """If the names of fields don't match, an error is raised."""
         query = Query(
             "query",
-            20,
             ["db"],
             [QueryMetric("metric", ["label"])],
             'SELECT 1 AS foo, "bar" AS label',
